/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.tree;

import java.util.List;
import nl.cloudfarming.client.isobus.model.DeviceElement;
import nl.cloudfarming.client.isobus.model.IsobusPolygon;
import nl.cloudfarming.client.isobus.model.IsobusProduct;
import nl.cloudfarming.client.isobus.model.ProcessDataVariable;
import nl.cloudfarming.client.isobus.model.IsobusTreatmentZone;
import nl.cloudfarming.client.isobus.model.ValuePresentation;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle.Messages;

/**
 * Child factory for children of {@link IsobusTreatmentZone} Possible children
 * are {@link ProcessDataVariable} and {@link IsobusPolygon}
 *
 * @author Gerben Feenstra
 */
@Messages({
    "isobus project taskdata node name process data variable=Process data variable ({0})",})
public class TreatmentZoneChildFactory extends ChildFactory<Node> {

    private static final String PROCESS_DATA_VARIABLE_NODE_ICON = "";
    private final IsobusTreatmentZone treatmentZone;

    public TreatmentZoneChildFactory(IsobusTreatmentZone treatmentZone) {
        this.treatmentZone = treatmentZone;
    }

    @Override
    protected boolean createKeys(List<Node> toPopulate) {
        // ProcessDataVariable (PDV)
        for (ProcessDataVariable processDataVariable : treatmentZone.getPDV()) {
            String displayName = Bundle.isobus_project_taskdata_node_name_process_data_variable(processDataVariable.getProcessDataValue());
            toPopulate.add(new IsobusAggregateNode(Children.create(new ProcessDataVariableChildFactory(processDataVariable), true), PROCESS_DATA_VARIABLE_NODE_ICON, displayName));
        }

        // Polygon (PLN)
        for (IsobusPolygon polygon : treatmentZone.getPLN()) {
            toPopulate.add(new PolygonNode(polygon));
        }
        return true;
    }

    @Override
    protected Node createNodeForKey(Node node) {
        return node;
    }

    /**
     * <p>Child factory for the {@link ProcessDataVariable}</p> <p>Possible
     * children are {@link IsobusProduct}, {@link DeviceElement}, {@link ValuePresentation}
     * and {@link ProcessDataVariable}</p>
     */
    static class ProcessDataVariableChildFactory extends ChildFactory<Node> {

        private ProcessDataVariable processDataVariable;

        public ProcessDataVariableChildFactory(ProcessDataVariable processDataVariable) {
            this.processDataVariable = processDataVariable;
        }

        @Override
        protected boolean createKeys(List<Node> toPopulate) {
            //Product (PDT)
            IsobusProduct isobusProduct = (IsobusProduct) processDataVariable.getProductIdRef();
            if (isobusProduct != null) {
                toPopulate.add(new ProductNode(isobusProduct));
            }

            //DeviceElement (DET)
            DeviceElement deviceElement = (DeviceElement) processDataVariable.getDeviceElementIdRef();
            if (deviceElement != null) {
                toPopulate.add(new DeviceElementNode(deviceElement));
            }

            //ValuePresentation (VPN)
            ValuePresentation presentation = (ValuePresentation) processDataVariable.getValuePresentationIdRef();
            if (presentation != null) {
                toPopulate.add(new ValuePresentationNode(presentation));
            }

            //ProcessDataVariable (PDV)
            for (ProcessDataVariable dataVariable : processDataVariable.getPDV()) {
                String displayName = Bundle.isobus_project_taskdata_node_name_process_data_variable(dataVariable.getProcessDataValue());
                toPopulate.add(new IsobusAggregateNode(Children.create(new ProcessDataVariableChildFactory(dataVariable), true), PROCESS_DATA_VARIABLE_NODE_ICON, displayName));
            }
            
            return true;
        }

        @Override
        protected Node createNodeForKey(Node node) {
            return node;
        }
    }
}
